@@ -17,22 +17,39 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <gio/gio.h>
+#include <glib-object.h>
+#include <glib.h>
 #include <glib/gi18n.h>
+#include <mpv/client.h>
+#include <string.h>
 
 #include "gmpv_actionctl.h"
 #include "gmpv_playlist_widget.h"
 #include "gmpv_def.h"
-#include "gmpv_mpv_obj.h"
+#include "gmpv_mpv.h"
+#include "gmpv_mpv_wrapper.h"
+#include "gmpv_main_window.h"
 #include "gmpv_playlist.h"
 #include "gmpv_open_loc_dialog.h"
 #include "gmpv_pref_dialog.h"
 #include "gmpv_common.h"
+#include "gmpv_control_box.h"
 #include "gmpv_authors.h"
 
 #if GTK_CHECK_VERSION(3, 20, 0)
 #include "gmpv_shortcuts_window.h"
 #endif
 
+static void open_dialog_response_handler(	GtkDialog *dialog,
+						gint response_id,
+						gpointer data );
+static void open_location_dialog_response_handler(	GtkDialog *dialog,
+							gint response_id,
+							gpointer data );
+static void preferences_dialog_response_handler(	GtkDialog *dialog,
+							gint response_id,
+							gpointer data );
 static void show_open_dialog_handler(	GSimpleAction *action,
 					GVariant *param,
 					gpointer data );
@@ -79,6 +96,159 @@ static void show_about_dialog_handler(	GSimpleAction *action,
 					GVariant *param,
 					gpointer data );
 
+static void open_dialog_response_handler(	GtkDialog *dialog,
+						gint response_id,
+						gpointer data )
+{
+	GPtrArray *args = data;
+	GmpvApplication *app = g_ptr_array_index(args, 0);
+	gboolean *append = g_ptr_array_index(args, 1);
+
+	if(response_id == GTK_RESPONSE_ACCEPT)
+	{
+		GtkFileChooser *file_chooser;
+		GSList *uri_slist;
+		GSList *uri;
+		GSettings *main_config;
+		GSettings *win_config;
+		gboolean last_folder_enable;
+		gsize uri_list_size;
+		const gchar **uri_list;
+		gint i;
+
+		file_chooser = GTK_FILE_CHOOSER(dialog);
+		uri_slist = gtk_file_chooser_get_filenames(file_chooser);
+		uri = uri_slist;
+		main_config = g_settings_new(CONFIG_ROOT);
+		win_config = g_settings_new(CONFIG_WIN_STATE);
+		last_folder_enable =	g_settings_get_boolean
+					(main_config, "last-folder-enable");
+		uri_list_size =	sizeof(gchar **)*(g_slist_length(uri_slist)+1);
+		uri_list = g_malloc(uri_list_size);
+
+		for(i = 0; uri; i++)
+		{
+			uri_list[i] = uri->data;
+			uri = g_slist_next(uri);
+		}
+
+		uri_list[i] = NULL;
+
+		if(uri_slist)
+		{
+			GmpvMpv *mpv = gmpv_application_get_mpv(app);
+
+			gmpv_mpv_load_list(mpv, uri_list, *append, TRUE);
+		}
+
+		if(last_folder_enable)
+		{
+			gchar *last_folder_uri
+				= gtk_file_chooser_get_current_folder_uri
+					(file_chooser);
+
+			g_settings_set_string(	win_config,
+						"last-folder-uri",
+						last_folder_uri?:"" );
+
+			g_free(last_folder_uri);
+		}
+
+		g_free(uri_list);
+		g_slist_free_full(uri_slist, g_free);
+		g_object_unref(main_config);
+		g_object_unref(win_config);
+	}
+
+	g_free(append);
+	g_ptr_array_free(args, TRUE);
+	gmpv_file_chooser_destroy(dialog);
+}
+
+static void open_location_dialog_response_handler(	GtkDialog *dialog,
+							gint response_id,
+							gpointer data )
+{
+	GPtrArray *args = data;
+	gboolean *append = g_ptr_array_index(args, 1);
+
+	if(response_id == GTK_RESPONSE_ACCEPT)
+	{
+		GmpvApplication *app;
+		GmpvMpv *mpv;
+		const gchar *loc_str;
+
+		app = g_ptr_array_index(args, 0);
+		mpv = gmpv_application_get_mpv(app);
+		loc_str =	gmpv_open_loc_dialog_get_string
+				(GMPV_OPEN_LOC_DIALOG(dialog));
+
+		gmpv_mpv_set_property_flag(mpv, "pause", FALSE);
+		gmpv_mpv_load(mpv, loc_str, *append, TRUE);
+	}
+
+	g_free(append);
+	g_ptr_array_free(args, TRUE);
+	gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+static void preferences_dialog_response_handler(	GtkDialog *dialog,
+							gint response_id,
+							gpointer data )
+{
+	if(response_id == GTK_RESPONSE_ACCEPT)
+	{
+		GmpvApplication *app;
+		GmpvMainWindow *wnd;
+		GmpvMpv *mpv;
+		GSettings *settings;
+		gboolean csd_enable;
+		gboolean dark_theme_enable;
+
+		app = data;
+		wnd = gmpv_application_get_main_window(app);
+		mpv = gmpv_application_get_mpv(app);
+		settings = g_settings_new(CONFIG_ROOT);
+		csd_enable = g_settings_get_boolean(settings, "csd-enable");
+		dark_theme_enable =	g_settings_get_boolean
+					(settings, "dark-theme-enable");
+
+		if(gmpv_main_window_get_csd_enabled(wnd) != csd_enable)
+		{
+			GtkWidget *dialog;
+
+			dialog =	gtk_message_dialog_new
+					(	GTK_WINDOW(wnd),
+						GTK_DIALOG_DESTROY_WITH_PARENT,
+						GTK_MESSAGE_INFO,
+						GTK_BUTTONS_OK,
+						_("Enabling or disabling "
+						"client-side decorations "
+						"requires restarting %s to "
+						"take effect." ),
+						g_get_application_name() );
+
+			g_signal_connect(	dialog,
+						"response",
+						G_CALLBACK(gtk_widget_destroy),
+						NULL );
+
+			gtk_widget_show_all(dialog);
+		}
+
+		g_object_set(	gtk_settings_get_default(),
+				"gtk-application-prefer-dark-theme",
+				dark_theme_enable,
+				NULL );
+
+		gmpv_mpv_reset(mpv);
+		gtk_widget_queue_draw(GTK_WIDGET(wnd));
+		g_object_unref(settings);
+	}
+
+	gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
 static void show_open_dialog_handler(	GSimpleAction *action,
 					GVariant *param,
 					gpointer data )
@@ -92,12 +262,13 @@ static void show_open_dialog_handler(	GSimpleAction *action,
 	GtkFileFilter *filter = NULL;
 	GmpvFileChooser *open_dialog = NULL;
 	gboolean last_folder_enable = FALSE;
-	gboolean append = FALSE;
+	gboolean *append = g_malloc(sizeof(gboolean));
+	GPtrArray *args = g_ptr_array_sized_new(2);
 
-	g_variant_get(param, "b", &append);
+	g_variant_get(param, "b", append);
 
 	wnd = gmpv_application_get_main_window(app);
-	open_dialog = gmpv_file_chooser_new(	append?
+	open_dialog = gmpv_file_chooser_new(	(*append)?
 						_("Add File to Playlist"):
 						_("Open File"),
 						GTK_WINDOW(wnd),
@@ -110,6 +281,14 @@ static void show_open_dialog_handler(	GSimpleAction *action,
 				(main_config, "last-folder-enable");
 	filter = gtk_file_filter_new();
 
+	g_ptr_array_add(args, app);
+	g_ptr_array_add(args, append);
+
+	g_signal_connect(	open_dialog,
+				"response",
+				G_CALLBACK(open_dialog_response_handler),
+				args );
+
 	gtk_file_filter_add_mime_type(filter, "video/*");
 	gtk_file_filter_add_mime_type(filter, "audio/*");
 	gtk_file_filter_add_mime_type(filter, "image/*");
@@ -142,52 +321,11 @@ static void show_open_dialog_handler(	GSimpleAction *action,
 	}
 
 	gtk_file_chooser_set_select_multiple(file_chooser, TRUE);
+	gmpv_file_chooser_set_modal(open_dialog, TRUE);
+	gmpv_file_chooser_show(open_dialog);
 
-	if(gmpv_file_chooser_run(open_dialog) == GTK_RESPONSE_ACCEPT)
-	{
-		GSList *uri_slist = gtk_file_chooser_get_filenames(file_chooser);
-		GSList *uri = uri_slist;
-		gsize uri_list_size =	sizeof(gchar **)*
-					(g_slist_length(uri_slist)+1);
-		const gchar **uri_list = g_malloc(uri_list_size);
-		gint i;
-
-		for(i = 0; uri; i++)
-		{
-			uri_list[i] = uri->data;
-			uri = g_slist_next(uri);
-		}
-
-		uri_list[i] = NULL;
-
-		if(uri_slist)
-		{
-			GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
-
-			gmpv_mpv_obj_load_list(mpv, uri_list, append, TRUE);
-		}
-
-		if(last_folder_enable)
-		{
-			gchar *last_folder_uri
-				= gtk_file_chooser_get_current_folder_uri
-					(file_chooser);
-
-			g_settings_set_string(	win_config,
-						"last-folder-uri",
-						last_folder_uri?:"" );
-
-			g_free(last_folder_uri);
-		}
-
-		g_free(uri_list);
-		g_slist_free_full(uri_slist, g_free);
-	}
-
-
-	gmpv_file_chooser_destroy(open_dialog);
-	g_clear_object(&main_config);
-	g_clear_object(&win_config);
+	g_object_unref(main_config);
+	g_object_unref(win_config);
 }
 
 static void show_open_location_dialog_handler(	GSimpleAction *action,
@@ -196,21 +334,26 @@ static void show_open_location_dialog_handler(	GSimpleAction *action,
 {
 	GmpvApplication *app = data;
 	GmpvMainWindow *wnd = gmpv_application_get_main_window(app);
-	GtkWidget *dlg = gmpv_open_loc_dialog_new(GTK_WINDOW(wnd));
+	GtkWidget *dlg = NULL;
+	gboolean *append = g_malloc(sizeof(gboolean));
+	GPtrArray *args = g_ptr_array_sized_new(2);
 
-	if(gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_ACCEPT)
-	{
-		const gchar *loc_str;
-		GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
+	g_variant_get(param, "b", append);
 
-		loc_str =	gmpv_open_loc_dialog_get_string
-				(GMPV_OPEN_LOC_DIALOG(dlg));
+	dlg = gmpv_open_loc_dialog_new(	GTK_WINDOW(wnd),
+					(*append)?
+					_("Add Location to Playlist"):
+					_("Open Location") );
+	g_ptr_array_add(args, app);
+	g_ptr_array_add(args, append);
 
-		gmpv_mpv_obj_set_property_flag(mpv, "pause", FALSE);
-		gmpv_mpv_obj_load(mpv, loc_str, FALSE, TRUE);
-	}
+	g_signal_connect
+		(	dlg,
+			"response",
+			G_CALLBACK(open_location_dialog_response_handler),
+			args );
 
-	gtk_widget_destroy(dlg);
+	gtk_widget_show_all(dlg);
 }
 
 static void toggle_loop_handler(	GSimpleAction *action,
@@ -218,11 +361,11 @@ static void toggle_loop_handler(	GSimpleAction *action,
 					gpointer data )
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
+	GmpvMpv *mpv = gmpv_application_get_mpv(app);
 	gboolean loop = g_variant_get_boolean(value);
 
 	g_simple_action_set_state(action, value);
-	gmpv_mpv_obj_set_property_string(mpv, "loop", loop?"inf":"no");
+	gmpv_mpv_set_property_string(mpv, "loop", loop?"inf":"no");
 }
 
 static void show_shortcuts_dialog_handler(	GSimpleAction *action,
@@ -266,7 +409,7 @@ static void save_playlist_handler(	GSimpleAction *action,
 					gpointer data )
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv;
+	GmpvMpv *mpv;
 	GmpvMainWindow *wnd;
 	GmpvPlaylist *playlist;
 	GtkTreeModel *model;
@@ -278,9 +421,9 @@ static void save_playlist_handler(	GSimpleAction *action,
 	GtkTreeIter iter;
 	gboolean rc;
 
-	mpv = gmpv_application_get_mpv_obj(app);
+	mpv = gmpv_application_get_mpv(app);
 	wnd = gmpv_application_get_main_window(app);
-	playlist = gmpv_mpv_obj_get_playlist(mpv);
+	playlist = gmpv_mpv_get_playlist(mpv);
 	model = GTK_TREE_MODEL(gmpv_playlist_get_store(playlist));
 	dest_file = NULL;
 	dest_stream = NULL;
@@ -379,60 +522,21 @@ static void show_preferences_dialog_handler(	GSimpleAction *action,
 {
 	GmpvApplication *app = data;
 	GmpvMainWindow *wnd = gmpv_application_get_main_window(app);
-	GSettings *settings = g_settings_new(CONFIG_ROOT);
-	GmpvPrefDialog *pref_dialog;
-	gboolean csd_enable_old;
+	GtkWidget *pref_dialog = gmpv_pref_dialog_new(GTK_WINDOW(wnd));
 
-	csd_enable_old = g_settings_get_boolean(settings, "csd-enable");
-	pref_dialog = GMPV_PREF_DIALOG(gmpv_pref_dialog_new(GTK_WINDOW(wnd)));
+	g_signal_connect_after(	pref_dialog,
+				"response",
+				G_CALLBACK(preferences_dialog_response_handler),
+				app );
 
-	if(gtk_dialog_run(GTK_DIALOG(pref_dialog)) == GTK_RESPONSE_ACCEPT)
-	{
-		GmpvMpvObj *mpv;
-		gboolean csd_enable;
-		gboolean dark_theme_enable;
-
-		mpv = gmpv_application_get_mpv_obj(app);
-		csd_enable = g_settings_get_boolean(settings, "csd-enable");
-		dark_theme_enable =	g_settings_get_boolean
-					(settings, "dark-theme-enable");
-
-		if(csd_enable_old != csd_enable)
-		{
-			GtkWidget *dialog;
-
-			dialog =	gtk_message_dialog_new
-					(	GTK_WINDOW(wnd),
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_INFO,
-						GTK_BUTTONS_OK,
-						_("Enabling or disabling "
-						"client-side decorations "
-						"requires restarting %s to "
-						"take effect." ),
-						g_get_application_name() );
-
-			gtk_dialog_run(GTK_DIALOG(dialog));
-			gtk_widget_destroy(dialog);
-		}
-
-		g_object_set(	gtk_settings_get_default(),
-				"gtk-application-prefer-dark-theme",
-				dark_theme_enable,
-				NULL );
-
-		gmpv_mpv_obj_reset(mpv);
-		gtk_widget_queue_draw(GTK_WIDGET(wnd));
-	}
-
-	gtk_widget_destroy(GTK_WIDGET(pref_dialog));
+	gtk_widget_show_all(pref_dialog);
 }
 
 static void quit_handler(	GSimpleAction *action,
 				GVariant *param,
 				gpointer data )
 {
-	quit(data);
+	gmpv_application_quit(data);
 }
 
 static void set_track_handler(	GSimpleAction *action,
@@ -440,7 +544,7 @@ static void set_track_handler(	GSimpleAction *action,
 				gpointer data )
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv;
+	GmpvMpv *mpv;
 	gint64 id;
 	gchar *name;
 	const gchar *mpv_prop;
@@ -466,15 +570,15 @@ static void set_track_handler(	GSimpleAction *action,
 		g_assert_not_reached();
 	}
 
-	mpv = gmpv_application_get_mpv_obj(app);
+	mpv = gmpv_application_get_mpv(app);
 
 	if(id > 0)
 	{
-		gmpv_mpv_obj_set_property(mpv, mpv_prop, MPV_FORMAT_INT64, &id);
+		gmpv_mpv_set_property(mpv, mpv_prop, MPV_FORMAT_INT64, &id);
 	}
 	else
 	{
-		gmpv_mpv_obj_set_property_string(mpv, mpv_prop, "no");
+		gmpv_mpv_set_property_string(mpv, mpv_prop, "no");
 	}
 }
 
@@ -526,7 +630,7 @@ static void load_track_handler(	GSimpleAction *action,
 	if(gtk_dialog_run(GTK_DIALOG(open_dialog)) == GTK_RESPONSE_ACCEPT)
 	{
 		const gchar *cmd[] = {cmd_name, NULL, NULL};
-		GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
+		GmpvMpv *mpv = gmpv_application_get_mpv(app);
 		GSList *uri_list = gtk_file_chooser_get_filenames(file_chooser);
 		GSList *uri = uri_list;
 
@@ -534,7 +638,7 @@ static void load_track_handler(	GSimpleAction *action,
 		{
 			cmd[1] = uri->data;
 
-			gmpv_mpv_obj_command(mpv, cmd);
+			gmpv_mpv_command(mpv, cmd);
 
 			uri = g_slist_next(uri);
 		}
@@ -621,7 +725,8 @@ void gmpv_actionctl_map_actions(GmpvApplication *app)
 			{.name = "show-preferences-dialog",
 			.activate = show_preferences_dialog_handler},
 			{.name = "show-open-location-dialog",
-			.activate = show_open_location_dialog_handler},
+			.activate = show_open_location_dialog_handler,
+			.parameter_type = "b"},
 			{.name = "toggle-loop",
 			.state = "false",
 			.change_state = toggle_loop_handler},
