@@ -20,7 +20,9 @@
 #ifndef ACTIONCTL_H
 #define ACTIONCTL_H
 
-#include "gmpv_common.h"
+#include <glib.h>
+
+#include "gmpv_application.h"
 
 G_BEGIN_DECLS
 
