@@ -19,6 +19,7 @@
 
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
+#include <glib-unix.h>
 #include <gdk/gdk.h>
 #include <locale.h>
 #include <epoxy/gl.h>
@@ -26,11 +27,14 @@
 #include "gmpv_application.h"
 #include "gmpv_control_box.h"
 #include "gmpv_actionctl.h"
+#include "gmpv_inputctl.h"
 #include "gmpv_playbackctl.h"
 #include "gmpv_playlist_widget.h"
 #include "gmpv_track.h"
+#include "gmpv_common.h"
 #include "gmpv_menu.h"
-#include "gmpv_mpv_obj.h"
+#include "gmpv_mpv.h"
+#include "gmpv_mpv_wrapper.h"
 #include "gmpv_video_area.h"
 #include "gmpv_def.h"
 #include "mpris/gmpv_mpris.h"
@@ -40,7 +44,7 @@ struct _GmpvApplication
 {
 	GtkApplication parent;
 	gboolean no_existing_session;
-	GmpvMpvObj *mpv;
+	GmpvMpv *mpv;
 	GQueue *action_queue;
 	gchar **files;
 	guint inhibit_cookie;
@@ -95,7 +99,8 @@ static gchar *strnjoinv(	const gchar *separator,
 static gboolean process_action(gpointer data);
 static void mpv_prop_change_handler(mpv_event_property *prop, gpointer data);
 static void mpv_event_handler(mpv_event *event, gpointer data);
-static void mpv_error_handler(GmpvMpvObj *mpv, const gchar *err, gpointer data);
+static void mpv_error_handler(GmpvMpv *mpv, const gchar *err, gpointer data);
+static gboolean shutdown_signal_handler(gpointer data);
 static void drag_data_handler(	GtkWidget *widget,
 				GdkDragContext *context,
 				gint x,
@@ -104,22 +109,9 @@ static void drag_data_handler(	GtkWidget *widget,
 				guint info,
 				guint time,
 				gpointer data);
-static gchar *get_full_keystr(guint keyval, guint state);
-static gboolean key_press_handler(	GtkWidget *widget,
+static gboolean window_state_handler(	GtkWidget *widget,
 					GdkEvent *event,
 					gpointer data );
-static gboolean key_release_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data );
-static gboolean mouse_button_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data );
-static gboolean mouse_move_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data );
-static gboolean scroll_handler(	GtkWidget *widget,
-				GdkEvent *event,
-				gpointer data );
 static gboolean queue_render(GtkGLArea *area);
 static void opengl_cb_update_callback(void *cb_ctx);
 static void set_playlist_pos(GmpvApplication *app, gint64 pos);
@@ -128,9 +120,6 @@ static gboolean load_files(	GmpvApplication* app,
 				const gchar **files,
 				gboolean append );
 static void connect_signals(GmpvApplication *app);
-static inline void add_accelerator(	GtkApplication *app,
-					const char *accel,
-					const char *action );
 static void gmpv_application_class_init(GmpvApplicationClass *klass);
 static void gmpv_application_init(GmpvApplication *app);
 
@@ -143,7 +132,7 @@ static gboolean vid_area_render_handler(	GtkGLArea *area,
 	GmpvApplication *app = data;
 	mpv_opengl_cb_context *opengl_ctx;
 
-	opengl_ctx = gmpv_mpv_obj_get_opengl_cb_context(app->mpv);
+	opengl_ctx = gmpv_mpv_get_opengl_cb_context(app->mpv);
 
 	if(opengl_ctx)
 	{
@@ -326,22 +315,13 @@ static void startup_handler(GApplication *gapp, gpointer data)
 
 	if(csd_enable)
 	{
-		gmpv_control_box_set_fullscreen_btn_visible
-			(control_box, FALSE);
+		gmpv_control_box_set_fullscreen_btn_visible(control_box, FALSE);
 	}
 
 	gmpv_control_box_set_chapter_enabled(control_box, FALSE);
 
 	wid = gmpv_video_area_get_xid(gmpv_main_window_get_video_area(app->gui));
-	app->mpv = gmpv_mpv_obj_new(app->playlist_store, wid);
-
-	if(csd_enable)
-	{
-		gmpv_control_box_set_fullscreen_btn_visible
-			(control_box, FALSE);
-	}
-
-	gmpv_control_box_set_chapter_enabled(control_box, FALSE);
+	app->mpv = gmpv_mpv_new(app->playlist_store, wid);
 
 	connect_signals(app);
 	gmpv_mpris_init(app);
@@ -364,10 +344,10 @@ static void activate_handler(GApplication *gapp, gpointer data)
 	gtk_window_present(GTK_WINDOW(GMPV_APPLICATION(data)->gui));
 }
 
-static void mpv_init_handler(GmpvMpvObj *mpv, gpointer data)
+static void mpv_init_handler(GmpvMpv *mpv, gpointer data)
 {
 	GmpvApplication *app = data;
-	gchar *current_vo = gmpv_mpv_obj_get_property_string(mpv, "current-vo");
+	gchar *current_vo = gmpv_mpv_get_property_string(mpv, "current-vo");
 	GmpvVideoArea *vid_area = gmpv_main_window_get_video_area(app->gui);
 
 	gmpv_video_area_set_use_opengl(vid_area, !current_vo);
@@ -383,7 +363,7 @@ static void mpv_init_handler(GmpvMpvObj *mpv, gpointer data)
 					app );
 
 		gtk_gl_area_make_current(gl_area);
-		gmpv_mpv_obj_init_gl(app->mpv);
+		gmpv_mpv_init_gl(app->mpv);
 	}
 
 	mpv_free(current_vo);
@@ -402,8 +382,6 @@ static void open_handler(	GApplication *gapp,
 
 	if(n_files > 0)
 	{
-		GmpvMpvObjState state;
-
 		app->files = g_malloc(sizeof(GFile *)*(gsize)(n_files+1));
 
 		for(i = 0; i < n_files; i++)
@@ -413,9 +391,7 @@ static void open_handler(	GApplication *gapp,
 
 		app->files[i] = NULL;
 
-		gmpv_mpv_obj_get_state(app->mpv, &state);
-
-		if(state.ready)
+		if(gmpv_mpv_get_state(app->mpv)->ready)
 		{
 			load_files(app, (const gchar **)app->files, append);
 		}
@@ -438,10 +414,12 @@ static gboolean draw_handler(GtkWidget *widget, cairo_t *cr, gpointer data)
 						NULL,
 						app );
 
-	gmpv_mpv_obj_initialize(app->mpv);
-	gmpv_mpv_obj_set_opengl_cb_callback
+	gmpv_mpv_initialize(app->mpv);
+	gmpv_main_window_set_geometry
+		(app->gui, gmpv_mpv_get_geometry(app->mpv));
+	gmpv_mpv_set_opengl_cb_callback
 		(app->mpv, opengl_cb_update_callback, app);
-	gmpv_mpv_obj_set_event_callback
+	gmpv_mpv_set_event_callback
 		(app->mpv, mpv_event_handler, app);
 
 	if(!app->files)
@@ -460,7 +438,7 @@ static gboolean delete_handler(	GtkWidget *widget,
 				GdkEvent *event,
 				gpointer data )
 {
-	quit(data);
+	gmpv_application_quit(data);
 
 	return TRUE;
 }
@@ -474,7 +452,7 @@ static void grab_handler(GtkWidget *widget, gboolean was_grabbed, gpointer data)
 		g_debug(	"Main window has been shadowed; "
 				"sending global keyup to mpv" );
 
-		gmpv_mpv_obj_command_string(app->mpv, "keyup");
+		gmpv_mpv_command_string(app->mpv, "keyup");
 	}
 }
 
@@ -491,14 +469,14 @@ static void playlist_row_deleted_handler(	GmpvPlaylistWidget *playlist,
 {
 	GmpvApplication *app = data;
 
-	if(gmpv_mpv_obj_is_loaded(app->mpv))
+	if(gmpv_mpv_get_state(app->mpv)->loaded)
 	{
 		const gchar *cmd[] = {"playlist_remove", NULL, NULL};
 		gchar *index_str = g_strdup_printf("%d", pos);
 
 		cmd[1] = index_str;
 
-		gmpv_mpv_obj_command(app->mpv, cmd);
+		gmpv_mpv_command(app->mpv, cmd);
 
 		g_free(index_str);
 	}
@@ -526,7 +504,7 @@ static void playlist_row_reodered_handler(	GmpvPlaylistWidget *playlist,
 	cmd[1] = src_str;
 	cmd[2] = dest_str;
 
-	gmpv_mpv_obj_command(app->mpv, cmd);
+	gmpv_mpv_command(app->mpv, cmd);
 
 	g_free(src_str);
 	g_free(dest_str);
@@ -584,7 +562,7 @@ static void update_track_list(GmpvApplication *app, mpv_node* track_list)
 			{"sid", "set-subtitle-track"},
 			{NULL, NULL} };
 
-	GmpvMpvObj *mpv = app->mpv;
+	GmpvMpv *mpv = app->mpv;
 	mpv_node_list *org_list = track_list->u.list;
 	GSList *audio_list = NULL;
 	GSList *video_list = NULL;
@@ -598,14 +576,14 @@ static void update_track_list(GmpvApplication *app, mpv_node* track_list)
 
 		action =	g_action_map_lookup_action
 				(G_ACTION_MAP(app), track_map[i].action_name);
-		buf =	gmpv_mpv_obj_get_property_string
+		buf =	gmpv_mpv_get_property_string
 			(mpv, track_map[i].prop_name);
 		val = g_ascii_strtoll(buf, NULL, 10);
 
 		g_simple_action_set_state
 			(G_SIMPLE_ACTION(action), g_variant_new_int64(val));
 
-		gmpv_mpv_obj_free(buf);
+		gmpv_mpv_free(buf);
 	}
 
 	for(gint i = 0; i < org_list->num; i++)
@@ -677,14 +655,9 @@ static gboolean process_action(gpointer data)
 static void mpv_prop_change_handler(mpv_event_property *prop, gpointer data)
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv;
-	GmpvControlBox *control_box;
-	GmpvMpvObjState state;
-
-	mpv = app->mpv;
-	control_box = gmpv_main_window_get_control_box(app->gui);
-
-	gmpv_mpv_obj_get_state(mpv, &state);
+	GmpvMpv *mpv = app->mpv;
+	GmpvControlBox *control_box = gmpv_main_window_get_control_box(app->gui);
+	const GmpvMpvState *state = gmpv_mpv_get_state(mpv);
 
 	if(g_strcmp0(prop->name, "pause") == 0)
 	{
@@ -703,17 +676,18 @@ static void mpv_prop_change_handler(mpv_event_property *prop, gpointer data)
 		update_track_list(app, prop->data);
 	}
 	else if(g_strcmp0(prop->name, "volume") == 0
-	&& (state.init_load || state.loaded))
+	&& (state->init_load || state->loaded))
 	{
 		gdouble volume = prop->data?*((double *)prop->data)/100.0:0;
 
 		gmpv_control_box_set_volume(control_box, volume);
 	}
-	else if(g_strcmp0(prop->name, "length") == 0 && prop->data)
+	else if(g_strcmp0(prop->name, "duration") == 0 && prop->data)
 	{
-		gdouble length = *((gdouble *) prop->data);
+		gdouble duration = *((gdouble *) prop->data);
 
-		gmpv_control_box_set_seek_bar_length(control_box, (gint)length);
+		gmpv_control_box_set_seek_bar_duration
+			(control_box, (gint)duration);
 	}
 	else if(g_strcmp0(prop->name, "media-title") == 0 && prop->data)
 	{
@@ -723,7 +697,7 @@ static void mpv_prop_change_handler(mpv_event_property *prop, gpointer data)
 	}
 	else if(g_strcmp0(prop->name, "playlist-pos") == 0 && prop->data)
 	{
-		GmpvPlaylist *playlist = gmpv_mpv_obj_get_playlist(mpv);
+		GmpvPlaylist *playlist = gmpv_mpv_get_playlist(mpv);
 		gint64 pos = *((gint64 *)prop->data);
 
 		gmpv_playlist_set_indicator_pos(playlist, (gint)pos);
@@ -745,23 +719,21 @@ static void mpv_prop_change_handler(mpv_event_property *prop, gpointer data)
 static void mpv_event_handler(mpv_event *event, gpointer data)
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv = app->mpv;
-	GmpvMpvObjState state;
-
-	gmpv_mpv_obj_get_state(mpv, &state);
+	GmpvMpv *mpv = app->mpv;
+	const GmpvMpvState *state = gmpv_mpv_get_state(mpv);
 
 	if(event->event_id == MPV_EVENT_VIDEO_RECONFIG)
 	{
-		gdouble autofit_ratio = gmpv_mpv_obj_get_autofit_ratio(app->mpv);
+		gdouble autofit_ratio = gmpv_mpv_get_autofit_ratio(app->mpv);
 
-		if(state.new_file && autofit_ratio > 0)
+		if(state->new_file && autofit_ratio > 0)
 		{
 			resize_window_to_fit(app, autofit_ratio);
 		}
 	}
 	else if(event->event_id == MPV_EVENT_PROPERTY_CHANGE)
 	{
-		if(state.loaded)
+		if(state->loaded)
 		{
 			mpv_prop_change_handler(event->data, data);
 		}
@@ -772,14 +744,14 @@ static void mpv_event_handler(mpv_event *event, gpointer data)
 		GmpvPlaylist *playlist;
 		gchar *title;
 		gint64 pos = -1;
-		gdouble length = 0;
+		gdouble duration = 0;
 
 		control_box = gmpv_main_window_get_control_box(app->gui);
-		playlist = gmpv_mpv_obj_get_playlist(mpv);
+		playlist = gmpv_mpv_get_playlist(mpv);
 
 		if(app->target_playlist_pos != -1)
 		{
-			gmpv_mpv_obj_set_property
+			gmpv_mpv_set_property
 				(	mpv,
 					"playlist-pos",
 					MPV_FORMAT_INT64,
@@ -788,20 +760,20 @@ static void mpv_event_handler(mpv_event *event, gpointer data)
 			app->target_playlist_pos = -1;
 		}
 
-		gmpv_mpv_obj_get_property
+		gmpv_mpv_get_property
 			(mpv, "playlist-pos", MPV_FORMAT_INT64, &pos);
-		gmpv_mpv_obj_get_property
-			(mpv, "length", MPV_FORMAT_DOUBLE, &length);
+		gmpv_mpv_get_property
+			(mpv, "duration", MPV_FORMAT_DOUBLE, &duration);
 
-		title = gmpv_mpv_obj_get_property_string(mpv, "media-title");
+		title = gmpv_mpv_get_property_string(mpv, "media-title");
 
 		gmpv_control_box_set_enabled(control_box, TRUE);
-		gmpv_control_box_set_playing_state(control_box, !state.paused);
+		gmpv_control_box_set_playing_state(control_box, !state->paused);
 		gmpv_playlist_set_indicator_pos(playlist, (gint)pos);
-		gmpv_control_box_set_seek_bar_length(control_box, (gint)length);
+		gmpv_control_box_set_seek_bar_duration(control_box, (gint)duration);
 		gtk_window_set_title(GTK_WINDOW(app->gui), title);
 
-		gmpv_mpv_obj_free(title);
+		gmpv_mpv_free(title);
 	}
 	else if(event->event_id == MPV_EVENT_CLIENT_MESSAGE)
 	{
@@ -835,7 +807,7 @@ static void mpv_event_handler(mpv_event *event, gpointer data)
 	}
 	else if(event->event_id == MPV_EVENT_IDLE)
 	{
-		if(!state.init_load && !state.loaded)
+		if(!state->init_load && !state->loaded)
 		{
 			gmpv_main_window_reset(app->gui);
 			set_inhibit_idle(app, FALSE);
@@ -843,11 +815,11 @@ static void mpv_event_handler(mpv_event *event, gpointer data)
 	}
 	else if(event->event_id == MPV_EVENT_SHUTDOWN)
 	{
-		quit(app);
+		gmpv_application_quit(app);
 	}
 }
 
-static void mpv_error_handler(GmpvMpvObj *mpv, const gchar *err, gpointer data)
+static void mpv_error_handler(GmpvMpv *mpv, const gchar *err, gpointer data)
 {
 	GmpvApplication *app = data;
 
@@ -855,6 +827,14 @@ static void mpv_error_handler(GmpvMpvObj *mpv, const gchar *err, gpointer data)
 	show_error_dialog(app, NULL, err);
 }
 
+static gboolean shutdown_signal_handler(gpointer data)
+{
+	g_info("Shutdown signal received. Terminating...");
+	gmpv_application_quit(data);
+
+	return FALSE;
+}
+
 static void drag_data_handler(	GtkWidget *widget,
 				GdkDragContext *context,
 				gint x,
@@ -884,17 +864,17 @@ static void drag_data_handler(	GtkWidget *widget,
 	{
 		if(uri_list)
 		{
-			gmpv_mpv_obj_load_list(	app->mpv,
+			gmpv_mpv_load_list(	app->mpv,
 						(const gchar **)uri_list,
 						append,
 						TRUE );
 		}
 		else
 		{
-			gmpv_mpv_obj_load(	app->mpv,
-						(const gchar *)raw_data,
-						append,
-						TRUE );
+			gmpv_mpv_load(	app->mpv,
+					(const gchar *)raw_data,
+					append,
+					TRUE );
 		}
 	}
 
@@ -902,215 +882,33 @@ static void drag_data_handler(	GtkWidget *widget,
 	g_free(type);
 }
 
-static gchar *get_full_keystr(guint keyval, guint state)
-{
-	/* strlen("Ctrl+Alt+Shift+Meta+")+1 == 21 */
-	const gsize max_modstr_len = 21;
-	gchar modstr[max_modstr_len];
-	gboolean found = FALSE;
-	const gchar *keystr = gdk_keyval_name(keyval);
-	const gchar *keystrmap[] = KEYSTRING_MAP;
-	modstr[0] = '\0';
-
-	if((state&GDK_SHIFT_MASK) != 0)
-	{
-		g_strlcat(modstr, "Shift+", max_modstr_len);
-	}
-
-	if((state&GDK_CONTROL_MASK) != 0)
-	{
-		g_strlcat(modstr, "Ctrl+", max_modstr_len);
-	}
-
-	if((state&GDK_MOD1_MASK) != 0)
-	{
-		g_strlcat(modstr, "Alt+", max_modstr_len);
-	}
-
-	if((state&GDK_META_MASK) != 0 || (state&GDK_SUPER_MASK) != 0)
-	{
-		g_strlcat(modstr, "Meta+", max_modstr_len);
-	}
-
-	/* Translate GDK key name to mpv key name */
-	for(gint i = 0; !found && keystrmap[i]; i += 2)
-	{
-		gint rc = g_ascii_strncasecmp(	keystr,
-						keystrmap[i+1],
-						KEYSTRING_MAX_LEN );
-
-		if(rc == 0)
-		{
-			keystr = keystrmap[i];
-			found = TRUE;
-		}
-	}
-
-	return (strlen(keystr) > 0)?g_strconcat(modstr, keystr, NULL):NULL;
-}
-
-static gboolean key_press_handler(	GtkWidget *widget,
+static gboolean window_state_handler(	GtkWidget *widget,
 					GdkEvent *event,
 					gpointer data )
 {
-	const gchar *cmd[] = {"keydown", NULL, NULL};
-	GmpvApplication *app = data;
-	guint keyval = ((GdkEventKey*)event)->keyval;
-	guint state = ((GdkEventKey*)event)->state;
-	gchar *keystr = NULL;
+	GtkApplication *app = data;
+	GdkEventWindowState *state = (GdkEventWindowState *)event;
 
-	cmd[1] = keystr = get_full_keystr(keyval, state);
-
-	if(keystr)
+	if(state->changed_mask&GDK_WINDOW_STATE_FULLSCREEN)
 	{
-		g_debug("Sent '%s' key down to mpv", keystr);
-		gmpv_mpv_obj_command(app->mpv, cmd);
+		gboolean fullscreen;
+		GAction *action;
 
-		g_free(keystr);
+		fullscreen =	state->new_window_state&
+				GDK_WINDOW_STATE_FULLSCREEN;
+
+		action = g_action_map_lookup_action(	G_ACTION_MAP(app),
+							"toggle-playlist" );
+		g_object_set(action, "enabled", !fullscreen, NULL);
+
+		action = g_action_map_lookup_action(	G_ACTION_MAP(app),
+							"set-video-size" );
+		g_object_set(action, "enabled", !fullscreen, NULL);
 	}
 
 	return FALSE;
 }
 
-static gboolean key_release_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data )
-{
-	GmpvApplication *app = data;
-	guint keyval = ((GdkEventKey*)event)->keyval;
-	guint state = ((GdkEventKey*)event)->state;
-	gchar *keystr = NULL;
-	const gchar *cmd[] = {"keyup", NULL, NULL};
-
-	cmd[1] = keystr = get_full_keystr(keyval, state);
-
-	if(keystr)
-	{
-		g_debug("Sent '%s' key up to mpv", keystr);
-		gmpv_mpv_obj_command(app->mpv, cmd);
-
-		g_free(keystr);
-	}
-
-	return FALSE;
-}
-
-static gboolean mouse_button_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data )
-{
-	GdkEventButton *btn_event = (GdkEventButton *)event;
-
-	if(btn_event->type == GDK_BUTTON_PRESS
-	|| btn_event->type == GDK_BUTTON_RELEASE
-	|| btn_event->type == GDK_SCROLL)
-	{
-		GmpvApplication *app = data;
-		gchar *btn_str =	g_strdup_printf
-					("MOUSE_BTN%u", btn_event->button-1);
-		const gchar *type_str =	(btn_event->type == GDK_SCROLL)?
-					"keypress":
-					(btn_event->type == GDK_BUTTON_PRESS)?
-					"keydown":"keyup";
-		const gchar *key_cmd[] = {type_str, btn_str, NULL};
-
-		g_debug(	"Sent %s event for button %s to mpv",
-				type_str, btn_str );
-
-		gmpv_mpv_obj_command(app->mpv, key_cmd);
-
-		g_free(btn_str);
-	}
-
-	return TRUE;
-}
-
-static gboolean mouse_move_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data )
-{
-	GmpvApplication *app = data;
-	GdkEventMotion *motion_event = (GdkEventMotion *)event;
-	gchar *x_str = g_strdup_printf("%d", (gint)motion_event->x);
-	gchar *y_str = g_strdup_printf("%d", (gint)motion_event->y);
-	const gchar *cmd[] = {"mouse", x_str, y_str, NULL};
-
-	gmpv_mpv_obj_command(app->mpv, cmd);
-
-	g_free(x_str);
-	g_free(y_str);
-
-	return FALSE;
-}
-
-static gboolean scroll_handler(	GtkWidget *widget,
-				GdkEvent *event,
-				gpointer data )
-{
-	GdkEventScroll *scroll_event = (GdkEventScroll *)event;
-	guint button = 0;
-	gint count = 0;
-
-	/* Only one axis will be used at a time to prevent accidental activation
-	 * of commands bound to buttons associated with the other axis.
-	 */
-	if(ABS(scroll_event->delta_x) > ABS(scroll_event->delta_y))
-	{
-		count = (gint)ABS(scroll_event->delta_x);
-
-		if(scroll_event->delta_x <= -1)
-		{
-			button = 6;
-		}
-		else if(scroll_event->delta_x >= 1)
-		{
-			button = 7;
-		}
-	}
-	else
-	{
-		count = (gint)ABS(scroll_event->delta_y);
-
-		if(scroll_event->delta_y <= -1)
-		{
-			button = 4;
-		}
-		else if(scroll_event->delta_y >= 1)
-		{
-			button = 5;
-		}
-	}
-
-	if(button > 0)
-	{
-		GdkEventButton btn_event;
-
-		btn_event.type = scroll_event->type;
-		btn_event.window = scroll_event->window;
-		btn_event.send_event = scroll_event->send_event;
-		btn_event.time = scroll_event->time;
-		btn_event.x = scroll_event->x;
-		btn_event.y = scroll_event->y;
-		btn_event.axes = NULL;
-		btn_event.state = scroll_event->state;
-		btn_event.button = button;
-		btn_event.device = scroll_event->device;
-		btn_event.x_root = scroll_event->x_root;
-		btn_event.y_root = scroll_event->y_root;
-
-		for(gint i = 0; i < count; i++)
-		{
-			/* Not used */
-			gboolean rc;
-
-			g_signal_emit_by_name
-				(widget, "button-press-event", &btn_event, &rc);
-		}
-	}
-
-	return TRUE;
-}
-
 static gboolean queue_render(GtkGLArea *area)
 {
 	gtk_gl_area_queue_render(area);
@@ -1135,21 +933,21 @@ static void set_playlist_pos(GmpvApplication *app, gint64 pos)
 	gint64 mpv_pos;
 	gint rc;
 
-	rc = gmpv_mpv_obj_get_property(	app->mpv,
+	rc = gmpv_mpv_get_property(	app->mpv,
 					"playlist-pos",
 					MPV_FORMAT_INT64,
 					&mpv_pos );
 
-	gmpv_mpv_obj_set_property_flag(app->mpv, "pause", FALSE);
+	gmpv_mpv_set_property_flag(app->mpv, "pause", FALSE);
 
 	if(pos != mpv_pos)
 	{
 		if(rc >= 0)
 		{
-			gmpv_mpv_obj_set_property(	app->mpv,
-							"playlist-pos",
-							MPV_FORMAT_INT64,
-							&pos );
+			gmpv_mpv_set_property(	app->mpv,
+						"playlist-pos",
+						MPV_FORMAT_INT64,
+						&pos );
 		}
 		else
 		{
@@ -1182,13 +980,9 @@ static gboolean load_files(	GmpvApplication *app,
 				const gchar **files,
 				gboolean append )
 {
-	GmpvMpvObjState state;
-
-	gmpv_mpv_obj_get_state(app->mpv, &state);
-
 	if(files)
 	{
-		gmpv_mpv_obj_load_list(app->mpv, files, append, TRUE);
+		gmpv_mpv_load_list(app->mpv, files, append, TRUE);
 
 		g_strfreev(app->files);
 
@@ -1206,8 +1000,13 @@ static void connect_signals(GmpvApplication *app)
 	playlist = gmpv_main_window_get_playlist(app->gui);
 	video_area = gmpv_main_window_get_video_area(app->gui);
 
+	gmpv_inputctl_connect_signals(app);
 	gmpv_playbackctl_connect_signals(app);
 
+	g_unix_signal_add(SIGHUP, shutdown_signal_handler, app);
+	g_unix_signal_add(SIGINT, shutdown_signal_handler, app);
+	g_unix_signal_add(SIGTERM, shutdown_signal_handler, app);
+
 	g_signal_connect_after(	app->gui,
 				"draw",
 				G_CALLBACK(draw_handler),
@@ -1224,6 +1023,10 @@ static void connect_signals(GmpvApplication *app)
 				"drag-data-received",
 				G_CALLBACK(drag_data_handler),
 				app );
+	g_signal_connect(	app->gui,
+				"window-state-event",
+				G_CALLBACK(window_state_handler),
+				app );
 	g_signal_connect(	app->gui,
 				"grab-notify",
 				G_CALLBACK(grab_handler),
@@ -1232,30 +1035,6 @@ static void connect_signals(GmpvApplication *app)
 				"delete-event",
 				G_CALLBACK(delete_handler),
 				app );
-	g_signal_connect(	app->gui,
-				"key-press-event",
-				G_CALLBACK(key_press_handler),
-				app );
-	g_signal_connect(	app->gui,
-				"key-release-event",
-				G_CALLBACK(key_release_handler),
-				app );
-	g_signal_connect(	video_area,
-				"button-press-event",
-				G_CALLBACK(mouse_button_handler),
-				app );
-	g_signal_connect(	video_area,
-				"button-release-event",
-				G_CALLBACK(mouse_button_handler),
-				app );
-	g_signal_connect(	video_area,
-				"motion-notify-event",
-				G_CALLBACK(mouse_move_handler),
-				app );
-	g_signal_connect(	video_area,
-				"scroll-event",
-				G_CALLBACK(scroll_handler),
-				app );
 	g_signal_connect(	playlist,
 				"row-activated",
 				G_CALLBACK(playlist_row_activated_handler),
@@ -1274,15 +1053,6 @@ static void connect_signals(GmpvApplication *app)
 				app );
 }
 
-static inline void add_accelerator(	GtkApplication *app,
-					const char *accel,
-					const char *action )
-{
-	const char *const accels[] = {accel, NULL};
-
-	gtk_application_set_accels_for_action(app, action, accels);
-}
-
 static void gmpv_application_class_init(GmpvApplicationClass *klass)
 {
 }
@@ -1329,16 +1099,6 @@ static void gmpv_application_init(GmpvApplication *app)
 		(app, "open", G_CALLBACK(open_handler), app);
 }
 
-GmpvMainWindow *gmpv_application_get_main_window(GmpvApplication *app)
-{
-	return app->gui;
-}
-
-GmpvMpvObj *gmpv_application_get_mpv_obj(GmpvApplication *app)
-{
-	return app->mpv;
-}
-
 GmpvApplication *gmpv_application_new(gchar *id, GApplicationFlags flags)
 {
 	return GMPV_APPLICATION(g_object_new(	gmpv_application_get_type(),
@@ -1346,3 +1106,45 @@ GmpvApplication *gmpv_application_new(gchar *id, GApplicationFlags flags)
 						"flags", flags,
 						NULL ));
 }
+
+GmpvMainWindow *gmpv_application_get_main_window(GmpvApplication *app)
+{
+	return app->gui;
+}
+
+GmpvMpv *gmpv_application_get_mpv(GmpvApplication *app)
+{
+	return app->mpv;
+}
+
+void gmpv_application_quit(GmpvApplication *app)
+{
+	const gchar *cmd[] = {"quit", NULL};
+	GmpvMpv *mpv = gmpv_application_get_mpv(app);
+	GmpvMainWindow *wnd = gmpv_application_get_main_window(app);
+
+	if(gmpv_mpv_get_mpv_handle(mpv))
+	{
+		GmpvVideoArea *vid_area = gmpv_main_window_get_video_area(wnd);
+		GtkGLArea *gl_area = gmpv_video_area_get_gl_area(vid_area);
+
+		if(gtk_widget_get_realized(GTK_WIDGET(gl_area)))
+		{
+			/* Needed by gmpv_mpv_quit() to uninitialize
+			 * opengl-cb
+			 */
+			gtk_gl_area_make_current(GTK_GL_AREA(gl_area));
+		}
+
+		gmpv_mpv_command(mpv, cmd);
+		gmpv_mpv_quit(mpv);
+	}
+
+	if(!gmpv_main_window_get_fullscreen(wnd))
+	{
+		gmpv_main_window_save_state(wnd);
+	}
+
+	g_application_quit(G_APPLICATION(app));
+}
+
