@@ -17,17 +17,19 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <gio/gio.h>
 #include <gio/gsettingsbackend.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
 #include "gmpv_common.h"
 #include "gmpv_def.h"
-#include "gmpv_mpv_obj.h"
+#include "gmpv_mpv.h"
+#include "gmpv_mpv_wrapper.h"
 #include "gmpv_main_window.h"
-#include "gmpv_video_area.h"
 #include "gmpv_control_box.h"
-#include "gmpv_playlist_widget.h"
 
 gchar *get_config_dir_path(void)
 {
@@ -71,40 +73,6 @@ gchar *get_name_from_path(const gchar *path)
 	return basename?basename:g_strdup(path);
 }
 
-gboolean quit(gpointer data)
-{
-	const gchar *cmd[] = {"quit", NULL};
-	GmpvApplication *app = data;
-	GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
-	GmpvMainWindow *wnd = gmpv_application_get_main_window(app);
-
-	if(gmpv_mpv_obj_get_mpv_handle(mpv))
-	{
-		GmpvVideoArea *vid_area = gmpv_main_window_get_video_area(wnd);
-		GtkGLArea *gl_area = gmpv_video_area_get_gl_area(vid_area);
-
-		if(gtk_widget_get_realized(GTK_WIDGET(gl_area)))
-		{
-			/* Needed by gmpv_mpv_obj_quit() to uninitialize
-			 * opengl-cb
-			 */
-			gtk_gl_area_make_current(GTK_GL_AREA(gl_area));
-		}
-
-		gmpv_mpv_obj_command(mpv, cmd);
-		gmpv_mpv_obj_quit(mpv);
-	}
-
-	if(!gmpv_main_window_get_fullscreen(wnd))
-	{
-		gmpv_main_window_save_state(wnd);
-	}
-
-	g_application_quit(G_APPLICATION(app));
-
-	return FALSE;
-}
-
 void activate_action_string(GmpvApplication *app, const gchar *str)
 {
 	GActionMap *map = G_ACTION_MAP(app);
@@ -181,17 +149,14 @@ void migrate_config(GmpvApplication *app)
 gboolean update_seek_bar(gpointer data)
 {
 	GmpvApplication *app = data;
-	GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
-	mpv_handle *mpv_ctx = gmpv_mpv_obj_get_mpv_handle(mpv);
-	GmpvMpvObjState state;
+	GmpvMpv *mpv = gmpv_application_get_mpv(app);
+	mpv_handle *mpv_ctx = gmpv_mpv_get_mpv_handle(mpv);
 	gdouble time_pos = -1;
 	gint rc = -1;
 
-	gmpv_mpv_obj_get_state(mpv, &state);
-
-	if(state.loaded)
+	if(gmpv_mpv_get_state(mpv)->loaded)
 	{
-		rc = gmpv_mpv_obj_get_property(	mpv,
+		rc = gmpv_mpv_get_property(	mpv,
 						"time-pos",
 						MPV_FORMAT_DOUBLE,
 						&time_pos );
@@ -214,11 +179,11 @@ gboolean update_seek_bar(gpointer data)
 void seek(GmpvApplication *app, gdouble time)
 {
 	const gchar *cmd[] = {"seek", NULL, "absolute", NULL};
-	GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
+	GmpvMpv *mpv = gmpv_application_get_mpv(app);
 
-	if(!gmpv_mpv_obj_is_loaded(mpv))
+	if(!gmpv_mpv_get_state(mpv)->loaded)
 	{
-		gmpv_mpv_obj_load(mpv, NULL, FALSE, TRUE);
+		gmpv_mpv_load(mpv, NULL, FALSE, TRUE);
 	}
 	else
 	{
@@ -226,7 +191,7 @@ void seek(GmpvApplication *app, gdouble time)
 
 		cmd[1] = value_str;
 
-		gmpv_mpv_obj_command(mpv, cmd);
+		gmpv_mpv_command(mpv, cmd);
 		update_seek_bar(app);
 
 		g_free(value_str);
@@ -293,21 +258,21 @@ void show_error_dialog(GmpvApplication *app, const gchar *prefix, const gchar *m
 
 void resize_window_to_fit(GmpvApplication *app, gdouble multiplier)
 {
-	GmpvMpvObj *mpv = gmpv_application_get_mpv_obj(app);
-	gchar *video = gmpv_mpv_obj_get_property_string(mpv, "video");
+	GmpvMpv *mpv = gmpv_application_get_mpv(app);
+	gchar *video = gmpv_mpv_get_property_string(mpv, "video");
 	gint64 width;
 	gint64 height;
 	gint mpv_width_rc;
 	gint mpv_height_rc;
 
-	mpv_width_rc = gmpv_mpv_obj_get_property(	mpv,
-							"dwidth",
-							MPV_FORMAT_INT64,
-							&width );
-	mpv_height_rc = gmpv_mpv_obj_get_property(	mpv,
-							"dheight",
-							MPV_FORMAT_INT64,
-							&height );
+	mpv_width_rc = gmpv_mpv_get_property(	mpv,
+						"dwidth",
+						MPV_FORMAT_INT64,
+						&width );
+	mpv_height_rc = gmpv_mpv_get_property(	mpv,
+						"dheight",
+						MPV_FORMAT_INT64,
+						&height );
 
 	if(video
 	&& strncmp(video, "no", 3) != 0
@@ -327,5 +292,5 @@ void resize_window_to_fit(GmpvApplication *app, gdouble multiplier)
 		gmpv_main_window_resize_video_area(wnd, new_width, new_height);
 	}
 
-	gmpv_mpv_obj_free(video);
+	gmpv_mpv_free(video);
 }
