@@ -20,10 +20,9 @@
 #ifndef COMMON_H
 #define COMMON_H
 
-#include <mpv/client.h>
-#include <mpv/opengl_cb.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
-#include "gmpv_playlist.h"
 #include "gmpv_application.h"
 
 G_BEGIN_DECLS
@@ -32,7 +31,6 @@ gchar *get_config_dir_path(void);
 gchar *get_scripts_dir_path(void);
 gchar *get_path_from_uri(const gchar *uri);
 gchar *get_name_from_path(const gchar *path);
-gboolean quit(gpointer data);
 gboolean update_seek_bar(gpointer data);
 void activate_action_string(GmpvApplication *app, const gchar *str);
 void migrate_config(GmpvApplication *app);
@@ -46,12 +44,16 @@ void load_keybind(	GmpvApplication *app,
 #if GTK_CHECK_VERSION(3, 19, 7)
 #define GmpvFileChooser GtkFileChooserNative
 #define gmpv_file_chooser_destroy(x) gtk_native_dialog_destroy(GTK_NATIVE_DIALOG(x))
+#define gmpv_file_chooser_show(x) gtk_native_dialog_show(GTK_NATIVE_DIALOG(x))
+#define gmpv_file_chooser_set_modal(x, y) gtk_native_dialog_set_modal(GTK_NATIVE_DIALOG(x), y)
 #define gmpv_file_chooser_run(x) gtk_native_dialog_run(GTK_NATIVE_DIALOG(x))
 #define gmpv_file_chooser_new(title,parent,action,accept,cancel) \
 	gtk_file_chooser_native_new(title,parent,action,accept,cancel)
 #else
 #define GmpvFileChooser GtkWidget
 #define gmpv_file_chooser_destroy(x) gtk_widget_destroy(GTK_WIDGET(x))
+#define gmpv_file_chooser_show(x) gtk_widget_show_all(GTK_DIALOG(x))
+#define gmpv_file_chooser_set_modal(x, y) gtk_window_set_modal(GTK_WINDOW(x), y)
 #define gmpv_file_chooser_run(x) gtk_dialog_run(GTK_DIALOG(x))
 #define gmpv_file_chooser_new(title,parent,action,accept,cancel) \
 	gtk_file_chooser_dialog_new(	title,parent,action,\
