@@ -17,18 +17,17 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <glib/gi18n.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <gio/gio.h>
 #include <gdk/gdk.h>
 
-#ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/gdkwayland.h>
-#endif
-
 #include "gmpv_def.h"
 #include "gmpv_menu.h"
 #include "gmpv_application.h"
 #include "gmpv_playlist_widget.h"
 #include "gmpv_main_window.h"
+#include "gmpv_header_bar.h"
 #include "gmpv_control_box.h"
 #include "gmpv_video_area.h"
 
@@ -46,6 +45,7 @@ struct _GmpvMainWindow
 	gint width_offset;
 	gint height_offset;
 	gint resize_target[2];
+	gboolean csd;
 	gboolean fullscreen;
 	gboolean playlist_visible;
 	gboolean playlist_first_toggle;
@@ -53,9 +53,6 @@ struct _GmpvMainWindow
 	gint playlist_width;
 	guint timeout_tag;
 	GtkWidget *header_bar;
-	GtkWidget *open_hdr_btn;
-	GtkWidget *fullscreen_hdr_btn;
-	GtkWidget *menu_hdr_btn;
 	GtkWidget *main_box;
 	GtkWidget *vid_area_paned;
 	GtkWidget *vid_area;
@@ -81,6 +78,7 @@ static void resize_video_area_finalize(	GtkWidget *widget,
 					GdkRectangle *allocation,
 					gpointer data );
 static gboolean resize_to_target(gpointer data);
+static void set_window_pos(GmpvMainWindow *wnd, const GmpvGeometry *geom);
 
 G_DEFINE_TYPE(GmpvMainWindow, gmpv_main_window, GTK_TYPE_APPLICATION_WINDOW)
 
@@ -193,6 +191,69 @@ static gboolean resize_to_target(gpointer data)
 	return FALSE;
 }
 
+static void set_window_pos(GmpvMainWindow *wnd, const GmpvGeometry *geom)
+{
+	GmpvControlBox *box;
+	GdkScreen *screen;
+	gint screen_dim[2];
+	gint window_dim[2];
+	gboolean flip[2];
+	GValue geom_pos[2];
+	gint64 pos[2];
+
+	g_assert(geom);
+
+	box = gmpv_main_window_get_control_box(wnd);
+	screen = gdk_screen_get_default();
+	screen_dim[0] = gdk_screen_get_width(screen);
+	screen_dim[1] = gdk_screen_get_height(screen);
+	window_dim[0] = 0;
+	window_dim[1] = 0;
+	geom_pos[0] = geom->x;
+	geom_pos[1] = geom->y;
+	flip[0] = geom->flags&GMPV_GEOMETRY_FLIP_X;
+	flip[1] = geom->flags&GMPV_GEOMETRY_FLIP_Y;
+	pos[0] = 0;
+	pos[1] = 0;
+
+	gtk_window_get_size(GTK_WINDOW(wnd), &window_dim[0], &window_dim[1]);
+
+	for(gint i = 0; i < 2; i++)
+	{
+		GType type = G_VALUE_TYPE(&geom_pos[i]);
+
+		if(type == G_TYPE_INT64)
+		{
+			gint64 value =	g_value_get_int64(&geom_pos[i]);
+
+			pos[i] =	flip[i]?
+					screen_dim[i]-window_dim[i]-value:
+					value;
+		}
+		else if(type == G_TYPE_DOUBLE)
+		{
+			gdouble value =	g_value_get_double(&geom_pos[i]);
+
+			pos[i] = (gint64)(value*(screen_dim[i]-window_dim[i]));
+		}
+		else
+		{
+			g_assert_not_reached();
+		}
+	}
+
+	/* Adjust the y-position to account for the height of
+	 * the control box.
+	 */
+	if(flip[1] && gtk_widget_get_visible(GTK_WIDGET(box)))
+	{
+		pos[1] -=	gtk_widget_get_allocated_height
+				(GTK_WIDGET(box));
+	}
+
+	gtk_window_move(GTK_WINDOW(wnd), (gint)pos[0], (gint)pos[1]);
+}
+
 static void gmpv_main_window_class_init(GmpvMainWindowClass *klass)
 {
 	GObjectClass *obj_class = G_OBJECT_CLASS(klass);
@@ -213,15 +274,13 @@ static void gmpv_main_window_class_init(GmpvMainWindowClass *klass)
 
 static void gmpv_main_window_init(GmpvMainWindow *wnd)
 {
+	wnd->csd = FALSE;
 	wnd->fullscreen = FALSE;
 	wnd->playlist_visible = FALSE;
 	wnd->pre_fs_playlist_visible = FALSE;
 	wnd->playlist_width = PLAYLIST_DEFAULT_WIDTH;
 	wnd->timeout_tag = 0;
-	wnd->header_bar = gtk_header_bar_new();
-	wnd->open_hdr_btn = NULL;
-	wnd->fullscreen_hdr_btn = NULL;
-	wnd->menu_hdr_btn = NULL;
+	wnd->header_bar = gmpv_header_bar_new();
 	wnd->main_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	wnd->vid_area_paned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
 	wnd->vid_area = gmpv_video_area_new();
@@ -235,9 +294,6 @@ static void gmpv_main_window_init(GmpvMainWindow *wnd)
 				GDK_ENTER_NOTIFY_MASK
 				|GDK_LEAVE_NOTIFY_MASK );
 
-	gtk_header_bar_set_show_close_button(	GTK_HEADER_BAR(wnd->header_bar),
-						TRUE );
-
 	gtk_window_set_title(GTK_WINDOW(wnd), g_get_application_name());
 
 	gtk_paned_set_position(	GTK_PANED(wnd->vid_area_paned),
@@ -281,85 +337,45 @@ void gmpv_main_window_set_fullscreen(GmpvMainWindow *wnd, gboolean fullscreen)
 {
 	if(fullscreen != wnd->fullscreen)
 	{
-		GmpvControlBox *control_box = GMPV_CONTROL_BOX(wnd->control_box);
 		GmpvVideoArea *vid_area = GMPV_VIDEO_AREA(wnd->vid_area);
-		GtkContainer* main_box = GTK_CONTAINER(wnd->main_box);
+		GtkContainer *main_box = GTK_CONTAINER(wnd->main_box);
 
 		if(fullscreen)
 		{
-			GdkScreen *screen;
-			GdkWindow *window;
-			GdkRectangle monitor_geom;
-			gint width;
-			gint monitor;
-
-			screen = gtk_window_get_screen(GTK_WINDOW(wnd));
-			window = gtk_widget_get_window(GTK_WIDGET(wnd));
-			monitor =	gdk_screen_get_monitor_at_window
-					(screen, window);
-
-			gdk_screen_get_monitor_geometry
-				(screen, monitor, &monitor_geom);
-
-			width = monitor_geom.width/2;
-
-			gtk_widget_set_size_request
-				(wnd->control_box, width, -1);
-
 			g_object_ref(wnd->control_box);
 			gtk_container_remove(main_box, wnd->control_box);
 			gmpv_video_area_set_control_box
 				(vid_area, wnd->control_box);
 			g_object_unref(wnd->control_box);
 
-			gmpv_video_area_set_fullscreen_state(vid_area, TRUE);
 			gtk_window_fullscreen(GTK_WINDOW(wnd));
 			gtk_window_present(GTK_WINDOW(wnd));
 
-			if(gmpv_main_window_get_csd_enabled(wnd))
-			{
-				gmpv_control_box_set_fullscreen_btn_visible
-					(control_box, TRUE);
-			}
-			else
-			{
-				gtk_application_window_set_show_menubar
-					(GTK_APPLICATION_WINDOW(wnd), FALSE);
-			}
-
 			wnd->pre_fs_playlist_visible = wnd->playlist_visible;
-			gtk_widget_set_visible(wnd->playlist, FALSE);
 		}
 		else
 		{
-			gtk_widget_set_halign(wnd->control_box, GTK_ALIGN_FILL);
-			gtk_widget_set_valign(wnd->control_box, GTK_ALIGN_FILL);
-			gtk_widget_set_size_request(wnd->control_box, -1, -1);
-
 			g_object_ref(wnd->control_box);
 			gmpv_video_area_set_control_box(vid_area, NULL);
 			gtk_container_add(main_box, wnd->control_box);
 			g_object_unref(wnd->control_box);
 
-			gmpv_video_area_set_fullscreen_state(vid_area, FALSE);
 			gtk_window_unfullscreen(GTK_WINDOW(wnd));
 
-			if(gmpv_main_window_get_csd_enabled(wnd))
-			{
-				gmpv_control_box_set_fullscreen_btn_visible
-					(control_box, FALSE);
-			}
-			else
-			{
-				gtk_application_window_set_show_menubar
-					(GTK_APPLICATION_WINDOW(wnd), TRUE);
-			}
-
 			wnd->playlist_visible = wnd->pre_fs_playlist_visible;
-			gtk_widget_set_visible
-				(wnd->playlist, wnd->pre_fs_playlist_visible);
 		}
 
+		if(!gmpv_main_window_get_csd_enabled(wnd))
+		{
+			gtk_application_window_set_show_menubar
+				(GTK_APPLICATION_WINDOW(wnd), !fullscreen);
+		}
+
+		gmpv_video_area_set_fullscreen_state(vid_area, fullscreen);
+		gtk_widget_set_visible(	wnd->playlist,
+					!fullscreen &&
+					wnd->pre_fs_playlist_visible );
+
 		wnd->fullscreen = fullscreen;
 	}
 }
@@ -459,6 +475,24 @@ void gmpv_main_window_load_state(GmpvMainWindow *wnd)
 	}
 }
 
+void gmpv_main_window_set_geometry(	GmpvMainWindow *wnd,
+					const GmpvGeometry *geom )
+{
+	if(geom)
+	{
+		if(!(geom->flags&GMPV_GEOMETRY_IGNORE_DIM))
+		{
+			gmpv_main_window_resize_video_area
+				(wnd, (gint)geom->width, (gint)geom->height);
+		}
+
+		if(!(geom->flags&GMPV_GEOMETRY_IGNORE_POS))
+		{
+			set_window_pos(wnd, geom);
+		}
+	}
+}
+
 void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
 						const GSList *audio_list,
 						const GSList *video_list,
@@ -466,14 +500,16 @@ void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
 {
 	if(gmpv_main_window_get_csd_enabled(wnd))
 	{
-		GMenu *menu = g_menu_new();
-
-		gmpv_menu_build_menu_btn
-			(menu, audio_list, video_list, sub_list);
-
-		gtk_menu_button_set_menu_model
-			(	GTK_MENU_BUTTON(wnd->menu_hdr_btn),
-				G_MENU_MODEL(menu) );
+		gmpv_header_bar_update_track_list
+			(	GMPV_HEADER_BAR(wnd->header_bar),
+				audio_list,
+				video_list,
+				sub_list );
+		gmpv_video_area_update_track_list
+			(	GMPV_VIDEO_AREA(wnd->vid_area),
+				audio_list,
+				video_list,
+				sub_list );
 	}
 	else
 	{
@@ -518,66 +554,8 @@ void gmpv_main_window_resize_video_area(	GmpvMainWindow *wnd,
 
 void gmpv_main_window_enable_csd(GmpvMainWindow *wnd)
 {
-	GMenu *menu_btn_menu;
-	GMenu *open_btn_menu;
-	GIcon *open_icon;
-	GIcon *fullscreen_icon;
-	GIcon *menu_icon;
-
-	open_btn_menu = g_menu_new();
-	menu_btn_menu = g_menu_new();
-
-	open_icon = g_themed_icon_new_with_default_fallbacks
-				("list-add-symbolic");
-	fullscreen_icon = g_themed_icon_new_with_default_fallbacks
-				("view-fullscreen-symbolic");
-	menu_icon = g_themed_icon_new_with_default_fallbacks
-				("open-menu-symbolic");
-
+	wnd->csd = TRUE;
 	wnd->playlist_width = PLAYLIST_DEFAULT_WIDTH;
-	wnd->open_hdr_btn = gtk_menu_button_new();
-	wnd->fullscreen_hdr_btn = gtk_button_new();
-	wnd->menu_hdr_btn = gtk_menu_button_new();
-
-	gmpv_menu_build_open_btn(open_btn_menu);
-	gmpv_menu_build_menu_btn(menu_btn_menu, NULL, NULL, NULL);
-
-	gtk_widget_set_can_focus(wnd->open_hdr_btn, FALSE);
-	gtk_widget_set_can_focus(wnd->fullscreen_hdr_btn, FALSE);
-	gtk_widget_set_can_focus(wnd->menu_hdr_btn, FALSE);
-
-	gtk_button_set_image
-		(	GTK_BUTTON(wnd->fullscreen_hdr_btn),
-			gtk_image_new_from_gicon
-				(fullscreen_icon, GTK_ICON_SIZE_MENU ));
-	gtk_button_set_image
-		(	GTK_BUTTON(wnd->open_hdr_btn),
-			gtk_image_new_from_gicon
-				(open_icon, GTK_ICON_SIZE_MENU ));
-	gtk_button_set_image
-		(	GTK_BUTTON(wnd->menu_hdr_btn),
-			gtk_image_new_from_gicon
-				(menu_icon, GTK_ICON_SIZE_MENU ));
-	gtk_menu_button_set_menu_model
-		(	GTK_MENU_BUTTON(wnd->open_hdr_btn),
-			G_MENU_MODEL(open_btn_menu) );
-	gtk_menu_button_set_menu_model
-		(	GTK_MENU_BUTTON(wnd->menu_hdr_btn),
-			G_MENU_MODEL(menu_btn_menu) );
-
-	gtk_header_bar_pack_start
-		(GTK_HEADER_BAR(wnd->header_bar), wnd->open_hdr_btn);
-	gtk_header_bar_pack_end
-		(GTK_HEADER_BAR(wnd->header_bar), wnd->menu_hdr_btn);
-	gtk_header_bar_pack_end
-		(GTK_HEADER_BAR(wnd->header_bar), wnd->fullscreen_hdr_btn);
-
-	gtk_widget_set_tooltip_text(	wnd->fullscreen_hdr_btn,
-					_("Toggle Fullscreen") );
-
-	gtk_actionable_set_action_name
-		(	GTK_ACTIONABLE(wnd->fullscreen_hdr_btn),
-			"app.toggle-fullscreen" );
 
 	gtk_paned_set_position(	GTK_PANED(wnd->vid_area_paned),
 				MAIN_WINDOW_DEFAULT_WIDTH
@@ -589,9 +567,7 @@ void gmpv_main_window_enable_csd(GmpvMainWindow *wnd)
 
 gboolean gmpv_main_window_get_csd_enabled(GmpvMainWindow *wnd)
 {
-	return	wnd->open_hdr_btn &&
-		wnd->fullscreen_hdr_btn &&
-		wnd->menu_hdr_btn;
+	return wnd->csd;
 }
 
 void gmpv_main_window_set_playlist_visible(	GmpvMainWindow *wnd,
