@@ -17,6 +17,9 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <gio/gio.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <glib/gi18n.h>
 
 #include "gmpv_menu.h"
@@ -188,7 +191,7 @@ void gmpv_menu_build_full(	GMenu *menu,
 	const GmpvMenuEntry entries[]
 		= {	{_("_File"), NULL, NULL},
 			{_("_Open"), "app.show-open-dialog(false)", NULL},
-			{_("Open _Location"), "app.show-open-location-dialog", NULL},
+			{_("Open _Location"), "app.show-open-location-dialog(false)", NULL},
 			{_("_Save Playlist"), "app.save-playlist", NULL},
 			{_("_Quit"), "app.quit", NULL},
 			{_("_Edit"), NULL, NULL},
@@ -252,7 +255,7 @@ void gmpv_menu_build_open_btn(GMenu *menu)
 	const GmpvMenuEntry entries[]
 		= {	{NULL, "", NULL},
 			{_("_Open"), "app.show-open-dialog(false)", NULL},
-			{_("Open _Location"), "app.show-open-location-dialog", NULL},
+			{_("Open _Location"), "app.show-open-location-dialog(false)", NULL},
 			{NULL, NULL, NULL} };
 
 	build_menu(menu, entries, TRUE);
