@@ -17,17 +17,61 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <gdk/gdk.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <string.h>
 #include <mpv/client.h>
 
 #include "gmpv_mpv_opt.h"
-#include "gmpv_mpv_obj.h"
-#include "gmpv_mpv_obj_private.h"
+#include "gmpv_mpv.h"
+#include "gmpv_mpv_private.h"
+#include "gmpv_mpv_wrapper.h"
+#include "gmpv_geometry.h"
 #include "gmpv_def.h"
 
-static void parse_dim_string(const gchar *geom_str, gint dim[2]);
+static gboolean parse_geom_token(const gchar **iter, GValue *value);
+static gboolean parse_dim_string(const gchar *geom_str, gint64 dim[2]);
+static gboolean parse_pos_token_prefix(const gchar **iter, gchar **prefix);
+static gboolean parse_pos_token(	const gchar **iter,
+					GValue *value,
+					gboolean *flip );
+static gboolean parse_pos_string(	const gchar *geom_str,
+					gboolean flip[2],
+					GValue pos[2] );
+static void parse_geom_string(	GmpvMpv *mpv,
+				const gchar *geom_str,
+				GmpvGeometry **geom );
 
-static void parse_dim_string(const gchar *geom_str, gint dim[2])
+static gboolean parse_geom_token(const gchar **iter, GValue *value)
+{
+	gchar *end = NULL;
+	gint64 token_value = g_ascii_strtoll(*iter, &end, 10);
+	gboolean rc = FALSE;
+
+	if(end)
+	{
+		if(*end == '%')
+		{
+			g_value_init(value, G_TYPE_DOUBLE);
+			g_value_set_double(value, (gdouble)token_value/100.0);
+
+			end++;
+		}
+		else
+		{
+			g_value_init(value, G_TYPE_INT64);
+			g_value_set_int64(value, token_value);
+		}
+
+		rc = !!end && *iter != end;
+		*iter = end;
+	}
+
+	return rc;
+}
+
+static gboolean parse_dim_string(const gchar *geom_str, gint64 dim[2])
 {
 	GdkScreen *screen = gdk_screen_get_default();
 	gint screen_dim[2] = {0, 0};
@@ -35,6 +79,8 @@ static void parse_dim_string(const gchar *geom_str, gint dim[2])
 	gchar **tokens = g_strsplit(geom_str, "x", 2);
 	gint i = -1;
 
+	dim[0] = -1;
+	dim[1] = -1;
 	screen_dim[0] = gdk_screen_get_width(screen);
 	screen_dim[1] = gdk_screen_get_height(screen);
 
@@ -65,9 +111,126 @@ static void parse_dim_string(const gchar *geom_str, gint dim[2])
 	}
 
 	g_strfreev(tokens);
+
+	return (dim[0] > 0 && dim[1] > 0);
 }
 
-void gmpv_mpv_opt_handle_autofit(GmpvMpvObj *mpv)
+static gboolean parse_pos_token_prefix(const gchar **iter, gchar **prefix)
+{
+	const gchar *start = *iter;
+	const gchar *end = *iter;
+	gboolean rc = FALSE;
+
+	while(*end && (*end == '+' || *end == '-'))
+	{
+		end++;
+	}
+
+	rc = (end-start <= 2);
+
+	if(rc)
+	{
+		if(end-start == 2)
+		{
+			end--;
+		}
+
+		*prefix = g_strndup(start, (gsize)(end-start));
+		*iter = end;
+	}
+
+	return rc;
+}
+
+static gboolean parse_pos_token(	const gchar **iter,
+					GValue *value,
+					gboolean *flip )
+{
+	gchar *prefix = NULL;
+	gboolean rc = TRUE;
+
+	rc = rc && parse_pos_token_prefix(iter, &prefix);
+	rc = rc && parse_geom_token(iter, value);
+	*flip = (prefix && *prefix == '-');
+
+	g_free(prefix);
+
+	return rc;
+}
+
+static gboolean parse_pos_string(	const gchar *geom_str,
+					gboolean flip[2],
+					GValue pos[2] )
+{
+	gboolean rc = TRUE;
+
+	rc = rc && parse_pos_token(&geom_str, &pos[0], &flip[0]);
+	rc = rc && parse_pos_token(&geom_str, &pos[1], &flip[1]);
+
+	return rc;
+}
+
+static void parse_geom_string(	GmpvMpv *mpv,
+				const gchar *geom_str,
+				GmpvGeometry **geom )
+{
+	gint64 dim[2] = {0, 0};
+	GValue pos[2] = {G_VALUE_INIT, G_VALUE_INIT};
+	gboolean flip[2] = {FALSE, FALSE};
+	gboolean dim_valid = FALSE;
+	gboolean pos_valid = FALSE;
+
+	if(!!geom_str)
+	{
+		*geom = g_new0(GmpvGeometry, 1);
+
+		if(geom_str[0] != '+' && geom_str[0] != '-')
+		{
+			dim_valid = parse_dim_string(geom_str, dim);
+		}
+
+		/* Move the beginning of the string to the 'position' section */
+		while(*geom_str && *geom_str != '+' && *geom_str != '-')
+		{
+			geom_str++;
+		}
+
+		pos_valid = parse_pos_string(geom_str, flip, pos);
+
+		(*geom)->flags |= !dim_valid*GMPV_GEOMETRY_IGNORE_DIM;
+		(*geom)->flags |= !pos_valid*GMPV_GEOMETRY_IGNORE_POS;
+		(*geom)->flags |= flip[0]*GMPV_GEOMETRY_FLIP_X;
+		(*geom)->flags |= flip[1]*GMPV_GEOMETRY_FLIP_Y;
+
+		(*geom)->x = pos[0];
+		(*geom)->y = pos[1];
+		(*geom)->width = dim[0];
+		(*geom)->height = dim[1];
+	}
+	else
+	{
+		*geom = NULL;
+
+		g_warning("Failed to parse geometry string: %s", geom_str);
+	}
+}
+
+void gmpv_mpv_opt_handle_geometry(GmpvMpv *mpv)
+{
+	gchar *opt_buf;
+
+	opt_buf  =	mpv_get_property_string
+			(mpv->mpv_ctx, "options/geometry");
+
+	if(opt_buf)
+	{
+		parse_geom_string(mpv, opt_buf, &mpv->geometry);
+
+		mpv_free(opt_buf);
+	}
+}
+
+void gmpv_mpv_opt_handle_autofit(GmpvMpv *mpv)
 {
 	gchar *scale_str = NULL;
 	gchar *autofit_str = NULL;
@@ -94,9 +257,9 @@ void gmpv_mpv_opt_handle_autofit(GmpvMpvObj *mpv)
 
 	if(scale_set || autofit_set || larger_set || smaller_set)
 	{
-		gint larger_dim[2] = {G_MAXINT, G_MAXINT};
-		gint smaller_dim[2] = {0, 0};
-		gint autofit_dim[2] = {0, 0};
+		gint64 larger_dim[2] = {G_MAXINT, G_MAXINT};
+		gint64 smaller_dim[2] = {0, 0};
+		gint64 autofit_dim[2] = {0, 0};
 		gint64 vid_dim[2];
 		gdouble ratio[2];
 		gdouble scale = 1;
@@ -157,16 +320,16 @@ void gmpv_mpv_opt_handle_autofit(GmpvMpvObj *mpv)
 			}
 			else
 			{
-				autofit_dim[0] = (gint)vid_dim[0];
-				autofit_dim[1] = (gint)vid_dim[1];
+				autofit_dim[0] = vid_dim[0];
+				autofit_dim[1] = vid_dim[1];
 			}
 
 			if(scale_set)
 			{
 				autofit_dim[0]
-					= (gint)(scale*(gdouble)autofit_dim[0]);
+					= (gint64)(scale*(gdouble)autofit_dim[0]);
 				autofit_dim[1]
-					= (gint)(scale*(gdouble)autofit_dim[1]);
+					= (gint64)(scale*(gdouble)autofit_dim[1]);
 			}
 		}
 
@@ -191,11 +354,15 @@ void gmpv_mpv_opt_handle_autofit(GmpvMpvObj *mpv)
 			}
 			else
 			{
-				g_debug(	"Target video area size: %dx%d",
+				g_debug(	"Target video area size: "
+						"%" G_GINT64_FORMAT
+						"x%" G_GINT64_FORMAT,
 						autofit_dim[0], autofit_dim[1] );
 
-				ratio[0] = autofit_dim[0]/(gdouble)vid_dim[0];
-				ratio[1] = autofit_dim[1]/(gdouble)vid_dim[1];
+				ratio[0] =	(gdouble)autofit_dim[0]/
+						(gdouble)vid_dim[0];
+				ratio[1] =	(gdouble)autofit_dim[1]/
+						(gdouble)vid_dim[1];
 			}
 		}
 
@@ -221,22 +388,22 @@ void gmpv_mpv_opt_handle_autofit(GmpvMpvObj *mpv)
 	mpv_free(smaller_str);
 }
 
-void gmpv_mpv_opt_handle_fs(GmpvMpvObj *mpv)
+void gmpv_mpv_opt_handle_fs(GmpvMpv *mpv)
 {
 	gchar *optbuf;
 
-	optbuf = gmpv_mpv_obj_get_property_string(mpv, "options/fs");
+	optbuf = gmpv_mpv_get_property_string(mpv, "options/fs");
 
 	if(g_strcmp0(optbuf, "yes") == 0)
 	{
-		gmpv_mpv_obj_command_string
-			(mpv, "script-message gmpv-action fullscreen_toggle");
+		gmpv_mpv_command_string
+			(mpv, "script-message gmpv-action toggle-fullscreen");
 	}
 
 	mpv_free(optbuf);
 }
 
-void gmpv_mpv_opt_handle_msg_level(GmpvMpvObj *mpv)
+void gmpv_mpv_opt_handle_msg_level(GmpvMpv *mpv)
 {
 	const struct
 	{
