@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 gnome-mpv
+ * Copyright (c) 2014, 2016 gnome-mpv
  *
  * This file is part of GNOME MPV.
  *
@@ -20,6 +20,8 @@
 #ifndef OPEN_LOC_DIALOG_H
 #define OPEN_LOC_DIALOG_H
 
+#include <glib.h>
+#include <glib-object.h>
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
@@ -28,7 +30,7 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE(GmpvOpenLocDialog, gmpv_open_loc_dialog, GMPV, OPEN_LOC_DIALOG, GtkDialog)
 
-GtkWidget *gmpv_open_loc_dialog_new(GtkWindow *parent);
+GtkWidget *gmpv_open_loc_dialog_new(GtkWindow *parent, const gchar *title);
 const gchar *gmpv_open_loc_dialog_get_string(GmpvOpenLocDialog *dlg);
 guint64 gmpv_open_loc_dialog_get_string_length(GmpvOpenLocDialog *dlg);
 
