@@ -17,6 +17,11 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <string.h>
+#include <cairo.h>
+#include <glib.h>
+#include <gio/gio.h>
+#include <gdk/gdk.h>
 #include <glib/gi18n.h>
 
 #include "gmpv_playlist_widget.h"
@@ -493,16 +498,23 @@ static gboolean mouse_press_handler(	GtkWidget *widget,
 	{
 		GMenu *menu;
 		GMenuItem *add_menu_item;
+		GMenuItem *add_loc_menu_item;
 		GMenuItem *loop_menu_item;
 		GtkWidget *ctx_menu;
 
 		menu = g_menu_new();
-		add_menu_item =	g_menu_item_new
-				(_("_Add…"), "app.show-open-dialog(true)");
-		loop_menu_item =	g_menu_item_new
-					(_("Loop"), "app.toggle-loop");
+		add_menu_item
+			=	g_menu_item_new
+				(	_("_Add…"),
+					"app.show-open-dialog(true)" );
+		add_loc_menu_item
+			=	g_menu_item_new
+				(	_("Add _Location…"),
+					"app.show-open-location-dialog(true)" );
+		loop_menu_item = g_menu_item_new(_("Loop"), "app.toggle-loop");
 
 		g_menu_append_item(menu, add_menu_item);
+		g_menu_append_item(menu, add_loc_menu_item);
 		g_menu_append_item(menu, loop_menu_item);
 		g_menu_freeze(menu);
 
