@@ -17,7 +17,9 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <glib.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "gmpv_shortcuts_window.h"
 
