@@ -20,6 +20,8 @@
 #ifndef VIDEO_AREA_H
 #define VIDEO_AREA_H
 
+#include <glib.h>
+#include <glib-object.h>
 #include <gtk/gtk.h>
 
 #define GMPV_TYPE_VIDEO_AREA (gmpv_video_area_get_type ())
@@ -27,6 +29,10 @@
 G_DECLARE_FINAL_TYPE(GmpvVideoArea, gmpv_video_area, GMPV, VIDEO_AREA, GtkOverlay)
 
 GtkWidget *gmpv_video_area_new(void);
+void gmpv_video_area_update_track_list(	GmpvVideoArea *hdr,
+					const GSList *audio_list,
+					const GSList *video_list,
+					const GSList *sub_list );
 void gmpv_video_area_set_fullscreen_state(	GmpvVideoArea *area,
 						gboolean fullscreen );
 void gmpv_video_area_set_control_box(	GmpvVideoArea *area,
